@@ -23,8 +23,8 @@ class Irradiance():
     import pandas as pd
     import numpy as np
     from pvmodule import PVGIS
-    from scipy.signal import savgol_filter
     import warnings
+    from threading import Thread
 
     warnings.simplefilter(action='ignore', category=pd.errors.PerformanceWarning)
 
@@ -40,89 +40,110 @@ class Irradiance():
     #WD10m: 10-m wind direction (0 = N
     #SP: Surface (air) pressure (Pa)
 
-    #Gb(i)  Gd(i)  Gr(i)  H_sun    T2m  WS10m  Int
-    inputs , data, metadata = PVGIS().retrieve_hourly(location.latitude ,
-                      location.longitude, 
-                      startyear = startyear, 
-                      endyear = endyear, 
-                      surface_tilt = panel_tilt, 
-                      surface_azimuth = azimuth, 
-                      components = 1)  
+    #month 	G(i) 	Gb(i) 	Gd(i) 	G(n) 	Gb(n) 	Gd(n) 	T2m 	WS10m
+    global df_res
+    global inputs
+    global metadata
+    df_res = pd.DataFrame()
+    def get_data(i):
+      global inputs
+      global metadata
+      global df_res
+      inputs, data , metadata = PVGIS().retrieve_daily(
+                    location.latitude, 
+                    location.longitude, 
+                    month= i, 
+                    angle = 0, 
+                    aspect = azimuth, 
+                    glob_2axis = 1)
+
+      df_res=  pd.concat([df_res, data])
+      
+      #df_res = df_res.concat(data)
+
+    threads = []
+    for i in range(1,13):
+      t1 = Thread(target=get_data, args=(i,))
+      threads.append(t1)
+
+    # Start all threads
+    for x in threads:
+        x.start()
+
+    # Wait for all of them to finish
+    for x in threads:
+            print(f'done: {x}')
+            x.join()
+
+    print("exit threads")
+    data = df_res.sort_values(by=['month'])
     
-    data.rename(columns = {'Gb(i)':'GHI',
-                           'Gd(i)':'DHI',
-                           'Gr(i)':'DNI',
+
+    data.rename(columns = {'G(i)':'GHI',
+                           'Gb(i)':'DHI',
+                           'Gb(n)':'DNI',
                            'T2m':'2m Air Temperature',
                            'WS10m':'10m Wind speed'
                           }, inplace = True)
-    data = data.drop(['Int','H_sun'], axis=1)
-
     
-    
-    #inputs ,data, metadata = PVGIS().retrieve_tmy(location.latitude,location.longitude)
 
 
-    #data.rename(columns = {'G(h)':'GHI',
-    #                       'Gd(h)':'DHI',
-    #                       'Gb(n)':'DNI',
-    #                       'RH':'Relative Humidity',
-    #                       'T2m':'2m Air Temperature',
-    #                       'WS10m':'10m Wind speed',
-    #                       'WD10m':'10m wind direction',
-    #                       'SP':'Air pressure',
-    #                      }, inplace = True)
-
-  
-
-    data['Month'] = data.index.strftime("%m").astype(float)
-    data['Day'] = data.index.strftime("%d").astype(float)
+    df_date = pd.DataFrame()
+    df_date['date'] = pd.date_range(f"{2030}-01-01 00:00:00", f"{2030}-12-31 23:00:00", freq='5T').to_frame()
+    data =  data.rename_axis('time').sort_values(by = ['month', 'time'], ascending = [True, True])
 
 
+    data['Time'] = pd.to_datetime(data.index).time
 
-    #new_index = data.index.map(lambda t: t.replace(year=2030))
-    #data=data.set_index(new_index)
-    #display(data)
-    if startyear != endyear:
-      data.index = pd.DatetimeIndex(data.index)
-      data.index = data.index + pd.DateOffset(year=2030)
-      data = data.groupby(data.index, as_index=True).mean()
-      tp_year = 2030
-    else:
-      tp_year = startyear
-    #data['Group_By'] = pd.to_datetime(data['Group_By'], format='2030-%m-%d %H:%M:%S')
+    print("logic 1")
 
 
+    df_date['Time'] = pd.to_datetime(df_date['date']).dt.time
+    df_date['Day'] = df_date.index.day
+    df_date['Month'] = df_date.index.month
 
+    final_df = pd.merge(data, df_date, how = 'inner', left_on = ['Time','month'], right_on = ['Time','Month'])
+    final_df.index = final_df['date']
+    final_df = final_df.sort_index()
 
-    #####
-
-
-    df_both = pd.date_range(f"{tp_year}-01-01 00:10:00", f"{tp_year}-12-31 23:10:00", freq='5T').to_frame()
+    df_both = pd.date_range(f"{2030}-01-01 00:00:00", f"{2030}-12-31 23:00:00", freq='5T').to_frame()
     df_both = df_both.drop([0], axis=1)
-    df_both = df_both.merge(data, left_index=True, right_index=True, how='left')
+    final_df = df_both.merge(final_df, left_index=True, right_index=True, how='left')
 
 
+    
 
 
+    print("logic 2")
+
         
-    df_both['Time_H'] = df_both.index.strftime("%H").astype(float)
-    df_both['Time_M'] = df_both.index.strftime("%M").astype(float)
-    df_both['Time_S'] = df_both.index.strftime("%S").astype(float)
+    final_df['Time_H'] = final_df.index.strftime("%H").astype(float)
+    final_df['Time_M'] = final_df.index.strftime("%M").astype(float)
+    final_df['Time_S'] = final_df.index.strftime("%S").astype(float)
 
-    df_both['Month']= df_both['Month'].fillna(method='ffill')
-    df_both['Day']= df_both['Day'].fillna(method='ffill')
+    final_df['Month']= final_df['Month'].fillna(method='ffill')
+    final_df['Day']= final_df['Day'].fillna(method='ffill')
+    final_df = final_df.drop(['date','Time','Gd(i)','G(n)','Gd(n)'], axis=1)
 
-    data = df_both.interpolate(method='polynomial', order=2)
+  
+    data = final_df
+
+    data = data.interpolate(method='polynomial', order=2)
     data['DHI'] = data['DHI'].round(0)
     data['GHI'] = data['GHI'].round(0)
     data['DNI'] = data['DNI'].round(0)
 
 
 
+
+
+
+    print("logic 3")
     DOY =  pd.DatetimeIndex(data.index.values).day_of_year
     data['Declination'] = (23.45*np.sin(np.deg2rad((360/365)*(284+DOY)))).values
 
     omega = abs((data['Time_H']*60+data['Time_M']+data['Time_S']/60)/4 - 180)
+    
     data = data.drop(['Time_H', 'Time_M','Time_S'], axis=1)
 
     data['Hour angle'] = omega
@@ -200,7 +221,6 @@ class Irradiance():
 
     G_Rear = GR_beam + GR_diffuse + GR_reflected
 
-    G_Rear = Irradiance()._PV_SOL_Rear(data, albedo, module, panel_distance, panel_tilt, azimuth,Elevation)
     data['G_Rear'] = G_Rear
     data['Total_G'] = G_Rear + G_front
     data[['GHI', 'DHI','DNI','G_Front','Total_G','G_Rear']] = data[['GHI', 'DHI','DNI','G_Front','Total_G','G_Rear']].clip(lower=0)
@@ -462,39 +482,6 @@ class Irradiance():
 
     return GR_reflected
 
-  def _PV_SOL_Rear(self, data, albedo, module, panel_distance, panel_tilt, azimuth, Elevation):
-    #https://help.valentin-software.com/pvsol/en/calculation/pv-modules/bifacial-modules/
-    import numpy as np
-
-    Xm = 0
-    h = Elevation
-    d = panel_distance
-    b = module['height']
-
-    x_start = Xm + - (h/ np.tan(panel_tilt*np.pi/180))
-    x_end = Xm + d + (h*(d - b * np.cos(panel_tilt*np.pi/180)))/(b*np.sin(panel_tilt*np.pi/180))
-
-    x = x_end - x_start
-
-    tan_delta_1v = np.arctan((h + b*np.sin(panel_tilt*np.pi/180))/(b * np.cos(panel_tilt*np.pi/180) - d - x))
-    tan_delta_1h =  np.arctan(h / x)
-
-    tan_delta_2v =  np.arctan((h + b*np.sin(panel_tilt*np.pi/180))/(x + b*np.cos(panel_tilt*np.pi/180)))
-    tan_delta_2h =   np.arctan(h / (x + d))
-
-    E_diffuse_test_point = data['DHI']*((tan_delta_1h - tan_delta_1v) + (tan_delta_2h + tan_delta_2v)) / (np.pi/2)
-
-    E_in_test_point = data['DHI'] + data['GHI']
-
-    E_out_test_point = albedo * E_in_test_point
-
-    E_rear_ground = E_out_test_point * (tan_delta_1h + tan_delta_2v)/(np.pi/180) * ((tan_delta_1h + tan_delta_2v)/(2)-panel_tilt*np.pi/180)
-
-    tany_next = (b*np.sin(panel_tilt*np.pi/180))/(d + b*np.cos(panel_tilt*np.pi/180))
-
-    E_rear_diffuse = data['DHI'] * (panel_tilt*np.pi/180 - tany_next)/(np.pi/180)
-
-    E_rear = E_rear_diffuse + E_rear_ground
 
     return E_rear
 
